local GH = GarrisonHelper
local L = GH.L
-- Frame and Events
local GarrisonHelper_Events = CreateFrame("Frame");
local events = {};
-- IDs
GH.armoryIDs = {
	["armoryNPC"] = {77377,79815},
	["ironHordeScraps"] = 113681,
	["scrapMeltdown"] = {38175,38188},
	["buildingID"] = 10,
	["weapon"] = 114128,
	["armor"] = 114745,
}

function GH:isNeededNPC()
	return GH:isArmoryEnabled() and (GH:isNPC(GH.armoryIDs["armoryNPC"][1]) or GH:isNPC(GH.armoryIDs["armoryNPC"][2])); 
end
function GH:isNeededQuest()
	return GetQuestID()==GH.armoryIDs["scrapMeltdown"][1] or GetQuestID()==GH.armoryIDs["scrapMeltdown"][2]; 
end
function GH:isArmoryEnabled()
	if type(GH.db.profile.bunkersupport)==nil then GH.db.profile.bunkersupport=true end
	return GH.db.profile.bunkersupport
end

function events.GOSSIP_SHOW()
	if GH:isNeededNPC() and GH:hasBuilding(GH.armoryIDs["buildingID"],3) then 
		if GetNumGossipAvailableQuests()==1 then
			if GetItemCount(GH.armoryIDs["ironHordeScraps"], true)>=25 then 	
				SelectGossipAvailableQuest(1);
			else
				GH:print(L["Not enough"] .. " " .. select(2,GetItemInfo(GH.armoryIDs["ironHordeScraps"])) .. " ("..GetItemCount(GH.armoryIDs["ironHordeScraps"], true).."/25)");
				if GH.db.profile.sound then
					PlaySound("igQuestFailed")
				end
			end
		end
		if GetNumGossipActiveQuests()==1 and GetItemCount(GH.armoryIDs["ironHordeScraps"], true)>=25 then
			SelectGossipActiveQuest(1);
		end
	end
end

function events.QUEST_DETAIL()
	if GH:isNeededQuest() and GetItemCount(GH.armoryIDs["ironHordeScraps"], true)>=25 then 
		AcceptQuest();
	end
end

function events.QUEST_COMPLETE()
	if GH:isNeededQuest() and GetItemCount(GH.armoryIDs["ironHordeScraps"], true)>=25 then 
		GH.db.profile.bunkertoken = GH.db.profile.bunkertoken or "smart"
		local o = 4
		if GH.db.profile.bunkertoken == "smart" then
			o = 4
		elseif GH.db.profile.bunkertoken == "random" then
			o = 3
		elseif GH.db.profile.bunkertoken == "weapon" then
			o = 2
		elseif GH.db.profile.bunkertoken == "armor" then
			o = 1
		end
		
		if o==3 then
			if math.random()>0.5 then o=1 else o=2 end
		elseif o==4 then
			o = GH:smartTokenChoice()
		end
		GetQuestReward(o);
	end
end

function events.LOOT_OPENED()
	if GH.db.profile.sound and GH:isArmoryEnabled() and GH:InsideGarrison() and GH:hasBuilding(GH.armoryIDs["buildingID"],3) and not (IsQuestFlaggedCompleted(GH.armoryIDs["scrapMeltdown"][2]) or IsQuestFlaggedCompleted(GH.armoryIDs["scrapMeltdown"][1])) then
		local loot_count = GetNumLootItems()
		GH.localisedIronHordeScrapsName = GH.localisedIronHordeScrapsName or GetItemInfo(GH.armoryIDs["ironHordeScraps"])
		if GH.localisedIronHordeScrapsName  then
			local c = GetItemCount(GH.armoryIDs["ironHordeScraps"], true)
			local texture, item, quantity, quality, locked, isQuestItem 
			for slot = 1, loot_count do
				texture, item, quantity, quality, locked, isQuestItem = GetLootSlotInfo(slot)
				if item and GH.localisedIronHordeScrapsName==item and c < 25 then
					if c+quantity<25 then
						PlaySound("igQuestFailed")
					else
						PlaySound("igQuestListComplete")
					end
					break
				end
			end
		end
	end
end

--TODO IDs !!!
function GH:smartTokenChoice()
		local o = 1
		local weaponCount = 3*GetItemCount(114128, true)+6*GetItemCount(114129, true)+9*GetItemCount(114131, true);
		local armorCount = 3*GetItemCount(114745, true)+6*GetItemCount(114808, true)+9*GetItemCount(114822, true);
		if weaponCount > armorCount then o=1 else o=2 end
		return o, weaponCount, armorCount
end

function GH:GetArmoryTooltip()
	if GH:isArmoryEnabled() and GH:hasBuilding(GH.armoryIDs["buildingID"],3) then
		local armoryName = select(2,C_Garrison.GetBuildingInfo(GH.armoryIDs["buildingID"]))
		local questName = GH:QuestTitleFromID(GH.armoryIDs["scrapMeltdown"][1]) or L["daily quest"]
		local tip = GH.tip
		tip:AddHeader("|cffF5DA81"..tostring(armoryName))
		tip:AddLine("   "..questName, ((IsQuestFlaggedCompleted(GH.armoryIDs["scrapMeltdown"][2]) or IsQuestFlaggedCompleted(GH.armoryIDs["scrapMeltdown"][1])) and "|cff00ff00"..L["completed"].."|r" or "|cffff0000"..L["not completed"].."|r") );
		if GH.db.profile.showdailyhelp then
			local itemTexture = "\124T" .. tostring(select(10,GetItemInfo(GH.armoryIDs["ironHordeScraps"])))  ..":24\124t";
			local c = GetItemCount(GH.armoryIDs["ironHordeScraps"], true)
			if c then
				tip:AddLine(" ", (c>=25 and "|cff00ff00" or "|cffff0000") .. c .. "|r "..itemTexture);
			end
			local weaponTexture = "\124T" .. tostring(select(10,GetItemInfo(GH.armoryIDs["weapon"])))  ..":24\124t";
			local armorTexture = "\124T" .. tostring(select(10,GetItemInfo(GH.armoryIDs["armor"])))  ..":24\124t";
			local _, weaponCount, armorCount = GH:smartTokenChoice()
			tip:AddLine(" ", weaponCount .. " " .. weaponTexture .. "     " .. armorCount .. " " .. armorTexture);
			--if c<25 then
				local name, texture, shipmentCapacity, shipmentsReady, shipmentsTotal, creationTime, duration, timeleftString, itemName, itemIcon, itemQuality, itemID = C_Garrison.GetLandingPageShipmentInfo(GH.armoryIDs["buildingID"]);
				if shipmentsReady then
					local approximatedCountMin =  floor(shipmentsReady *2)
					local approximatedCountMax =  floor(shipmentsReady *3)
					tip:AddLine("   " .. L["current shipment"]..":",  (shipmentsReady==0 and "" or (" " ..L["appr."].." ".. approximatedCountMin .. "-")) .. approximatedCountMax .. "|r "..itemTexture);
				end
			--end
		end
	end
end

function GH:GetArmoryTooltipExtension()
	local res = {}
	if GH:isArmoryEnabled() and GH:hasBuilding(GH.armoryIDs["buildingID"],3) then
		local questName = GH:QuestTitleFromID(GH.armoryIDs["scrapMeltdown"][1]) or L["daily quest"]
		local questcompleted = IsQuestFlaggedCompleted(GH.armoryIDs["scrapMeltdown"][2]) or IsQuestFlaggedCompleted(GH.armoryIDs["scrapMeltdown"][1])
		res[#res+1] = {left=" ",right=" "}
		res[#res+1] = {left=questName..":",right=( questcompleted and "|cff00ff00"..L["completed"].."|r" or "|cffff0000"..L["not completed"].."|r")}
		--local itemTexture = tostring(select(1,GetItemInfo(GH.armoryIDs["ironHordeScraps"])));
		local itemTexture = "\124T" .. tostring(select(10,GetItemInfo(GH.armoryIDs["ironHordeScraps"])))  ..":12\124t";
		local c = GetItemCount(GH.armoryIDs["ironHordeScraps"], true)
		if c and not questcompleted then
			res[#res+1] = {left=" ",right=(c>=25 and "|cff00ff00" or "|cffff0000") .. c .. "/25|r "..itemTexture}
		end
		
		local name, texture, shipmentCapacity, shipmentsReady, shipmentsTotal, creationTime, duration, timeleftString, itemName, itemIcon, itemQuality, itemID = C_Garrison.GetLandingPageShipmentInfo(GH.armoryIDs["buildingID"]);
		if shipmentsReady then
			local approximatedCountMin =  floor(shipmentsReady *2)
			local approximatedCountMax =  floor(shipmentsReady *3)
			res[#res+1] = {left=" ",right=" "}
			res[#res+1] = {left=L["current shipment"]..":",right=(shipmentsReady==0 and "" or (" " ..L["appr."].." ".. approximatedCountMin .. "-")) .. approximatedCountMax .. "|r "..itemTexture}
		end
		res[#res+1] = {left=" ",right=" "}
	end
	return res
end


function GH:GetArmoryOpenTaskCount()
	local openTask = 0
	local tasks = 0
	local timeUntilMax = 0
	local timeUntilMin = 0
	local scrapCount = GetItemCount(GH.armoryIDs["ironHordeScraps"], true)
	if GH:hasBuilding(GH.armoryIDs["buildingID"],3) then
		if GH:isArmoryEnabled() and not (IsQuestFlaggedCompleted(GH.armoryIDs["scrapMeltdown"][2]) or IsQuestFlaggedCompleted(GH.armoryIDs["scrapMeltdown"][1]))then
			openTask = 1
			if scrapCount < 25 then
				local maximal = floor(0.5+(25-scrapCount)/2) -- round to number of shipments needed to get enough scraps
				local minimal = floor(0.5+(25-scrapCount)/3)
				
				if minimal<=1 then minimal = 2 end
				if maximal<=1 then maximal = 2 end
				
				local name, texture, shipmentCapacity, shipmentsReady, shipmentsTotal, creationTime, duration, timeleftString, itemName, itemIcon, itemQuality, itemID = C_Garrison.GetLandingPageShipmentInfo(GH.armoryIDs["buildingID"]);
				if creationTime and duration and shipmentsTotal then
					shipmentsReady = shipmentsReady or 0
					maximal = maximal - shipmentsReady
					minimal = minimal - shipmentsReady
					shipmentsTotal = shipmentsTotal - shipmentsReady
					if (maximal - shipmentsTotal) <= 0 then
						timeUntilMax = creationTime+(duration*maximal)
					else
						timeUntilMax = -1
					end
					if (minimal - shipmentsTotal) <= 0 then
						timeUntilMin = creationTime+(duration*minimal)
					end
				else
					timeUntilMax = -1
				end
			end
		end
		tasks = 1
	end
	return openTask, tasks, timeUntilMin, timeUntilMax, scrapCount
end


function events.QUEST_PROGRESS()
	if GH:isNeededQuest() and GetItemCount(GH.armoryIDs["ironHordeScraps"], true)>=25 then
		if IsQuestCompletable() then CompleteQuest() end 
	end
end

function GH:ArmoryWelcome()
	if GH:isArmoryEnabled() and GH:hasBuilding(GH.armoryIDs["buildingID"],3) then
		local q = GH.armoryIDs["scrapMeltdown"][2];
		if UnitFactionGroup("player")=="Alliance" then q = GH.armoryIDs["scrapMeltdown"][1]; end
		if not IsQuestFlaggedCompleted(q) then
			local s= "";
			if GetItemCount(GH.armoryIDs["ironHordeScraps"], true)<25 then
				local l = select(2,GetItemInfo(GH.armoryIDs["ironHordeScraps"]))
				s = s .. " ("..L["note"]..": "..GetItemCount(GH.armoryIDs["ironHordeScraps"], true).."/25 "..l..")"
			end
			local questName = GH:QuestTitleFromID(GH.armoryIDs["scrapMeltdown"][1])
			if questName then
				GH:print(questName.." "..L["not completed today."]..s);
			end
		end
	end
end

-- register event listener
GarrisonHelper_Events:SetScript("OnEvent", function(self, event, ...) events[event](self, event, ...); end);
for event, func in pairs(events) do GarrisonHelper_Events:RegisterEvent(event); end


--[[
	Needed, because if cache is empty, this information have to be retrieved from server.
]]
function GH:PreLoadArmoryInfos()
	IsQuestFlaggedCompleted(GH.armoryIDs["scrapMeltdown"][2])
	IsQuestFlaggedCompleted(GH.armoryIDs["scrapMeltdown"][1])
	GetItemInfo(GH.armoryIDs["ironHordeScraps"])
	C_Garrison.GetBuildingInfo(GH.armoryIDs["buildingID"])
	GetItemInfo(GH.armoryIDs["armor"])
	GetItemInfo(GH.armoryIDs["weapon"])
	--for i=1,#GH.armoryIDs["armoryNPC"] do GH.armoryIDs["armoryNPC"][i] end
end
GH:PreLoadArmoryInfos() -- TODO move this to an event like addon loaded