local GH = GarrisonHelper
local L = GH.L
-- Frame and Events
local GarrisonHelper_Events = CreateFrame("Frame");
local events = {};
local addonName = ...
local formattedName = "\124r\124cFFFF7700<"..addonName..">\124r"
GH.dataobj = GH.LDB:NewDataObject(addonName, {label = formattedName, type = "data source", icon = "Interface\\Icons\\garrison_build.blp", text = nil})

if GH.dataobj then
	
	function GH.dataobj:OnEnter()
		GH:BrokerTooltip(self)
		--print("#1#"..tostring(self))
		GH:ScanGarrisonOverlayButton(self)
		--if not GH_OverlayButton then
			
		--end
	end
	
	function GH.dataobj:OnLeave()
		GameTooltip:Hide()
	end

	function GH.dataobj:OnClick(button)
		if button == "LeftButton" and IsShiftKeyDown() then
			--GH:print(button .. " and shift")
			SlashCmdList["GARRISONHELPER"]()
		elseif button == "LeftButton" then
			--GH:print(button)
			SlashCmdList["GARRISONHELPER"]()
		elseif button == "RightButton" then
			--GH:print(button)
			SlashCmdList["GARRISONHELPER"]()
		end
	end
end

function GH:BrokerTooltip(self)
	
	if GH.tip then GH.tip:Hide() end
	
	if GH.db.profile.tooltipincombat and InCombatLockdown() then return end
	
	GH.tip = GH.TT:Acquire("GH_Tooltip", 2, "LEFT", "RIGHT")
	
	local tip = GH.tip
	if self then
		tip:SmartAnchorTo(self)
		tip:SetAutoHideDelay(0.25, self)
	end
	tip:Clear()
	
	if #C_Garrison.GetBuildings(LE_GARRISON_TYPE_6_0) > 0 then
		GH:UpdateBrokerText()
		if GH.dataobj.text then tip:AddHeader(GH.dataobj.text) GH:AddTooltipSeparator() end
		GH:GetArmoryTooltip()
		GH:GetJewelcraftingTooltip()
		GH:GetScribeTooltip()
		GH:GetAlchemyTooltip()
	else
		tip:AddLine("\n"..L["No supported garrison buildings found."].."\n"..formattedName.." "..L["can't help this character."])
	end
		
	GH:GetDailyVisitorsTooltip()
	
	GH:GetOpenTasksTooltip()
	
	GH:AddTooltipSeparator()
	
	local info = {{left=L["Don't click here. Click on the now highlighted button."]}}
	if GH:InsideGarrison() then
		local lineIndex = tip:AddLine("\124TInterface\\TutorialFrame\\UI-TUTORIAL-FRAME:16:14:0:0:1000:1000:1:148:425:621\124t".." ".. L["to scan for NPCs in your garrison"])
		GH:AddMouseoverTooltip(tip, lineIndex, info, "OnMouseDown")
	end
	local lineIndex = tip:AddLine("\124TInterface\\TutorialFrame\\UI-TUTORIAL-FRAME:16:14:0:0:1000:1000:1:148:626:822\124t".." "..L["to open configuration"])
	
	GH:AddMouseoverTooltip(tip, lineIndex, info, "OnMouseDown")
	
	if self then tip:Show() end
end

function GH:AddMouseoverTooltip(tip, lineIndex, info, event)
	event = event or "OnEnter"
	tip:SetLineScript(lineIndex, event, function(self)
		if GH.tip2 then GH.tip2:Hide() end
		GH.tip2 = GH.TT:Acquire("GH_Tooltip2", 2, "LEFT", "RIGHT")
		local tip2 = GH.tip2
		if GH.tip then
			--tip2:SmartAnchorTo(GH.tip)
			tip2:ClearAllPoints()
			tip2:SetClampedToScreen(false)
			tip2:SetPoint("TOPRIGHT", GH.tip,"BOTTOMRIGHT")
			tip2:SetAutoHideDelay(0.25, GH.tip)
		elseif self then
			tip2:SmartAnchorTo(self)
			tip2:SetAutoHideDelay(0.25, self)
		end
		tip2:Clear()
		
		if type(info)=="string" then
			tip2:AddLine(info)
		elseif type(info)=="table" then
			for i=1,#info do
				tip2:AddLine(info[i].left,  info[i].right);
			end
		end
		tip2:Show()
		if event=="OnMouseDown" and GH_OverlayButton then
			if GH_OverlayButton and GH_OverlayButton.highlight then
				GH_OverlayButton.highlight:SetAlpha(1)
			end
		end
	end)
	tip:SetLineScript(lineIndex, "OnLeave", function()
		if GH.tip2 then GH.tip2:Hide(); end
		if GH_OverlayButton and GH_OverlayButton.highlight then
			GH_OverlayButton.highlight:SetAlpha(0)
		end
	end)
	
	--if self then tip:Show() end
end

function GH:AddTooltipSeparator()
	local tip = GH.tip
	if tip then
		local f = tip:GetFont()
		local fontName, fontHeight, fontFlags = f:GetFont()
		f:SetFont(fontName, 1, fontFlags)
		tip:SetFont(f)
		tip:AddLine(" ", " ")
		tip:AddSeparator(2,0.5,0.5,0.5)
		tip:AddLine(" ", " ")
		f:SetFont(fontName, fontHeight, fontFlags)
		tip:SetFont(f)
	end
end

function GH:UpdateBrokerText()
	if GH.dataobj then
		local c = GH:GetOpenTaskCount()
		GH.dataobj.text = c .. " " .. (c==1 and L["task"] or L["tasks"]) .. " "..L["open"]
	end
	GH:SaveTasks()
end

function GH:GetOpenTaskCount()
	return GH:GetArmoryOpenTaskCount()+GH:GetScribeOpenTaskCount()+GH:GetJewelcraftingOpenTaskCount()+GH:GetAlchemyOpenTaskCount()
end


function UpdateDisplay()
	if GH.dataobj then
		GH:UpdateBrokerText()
		C_Timer.After(10, UpdateDisplay)
	end
end
C_Timer.After(5, UpdateDisplay)



-- register event listener
GarrisonHelper_Events:SetScript("OnEvent", function(self, event, ...) GH:UpdateBrokerText() end);
GarrisonHelper_Events:RegisterEvent("QUEST_FINISHED");
GarrisonHelper_Events:RegisterEvent("BAG_UPDATE");
GarrisonHelper_Events:RegisterEvent("BAG_UPDATE_DALAYED");
