--[[
	TODO: if button was closed. maybe right click without closing quest dialog should bring button back
]]
local GH = GarrisonHelper
local L = GH.L
-- Frame and Events
local GarrisonHelper_Events = CreateFrame("Frame");
local events = {};

GH.jewelcraftingquests = {
	--[[
		quest name (not used anywhere), crafted item id, crafted item spell id, crafted item amount, required resources and amount
		unfortunately we have to note the required reagents, because opening a trade skill window is one click too much
	]]
	[37321] = {"Jewelcrafting Special Order: A Yellow Brighter Than Gold", 115803,170719,1,{[109118]=10,[109125]=2}},
	[37323] = {"Jewelcrafting Special Order: Blue the Shade of Sky and Sea", 115804,170720,2,{[109118]=20,[109124]=4}},
	[37319] = {"Jewelcrafting Special Order: Wedding Bands", 115993,170710,2,{[109119]=120}},
	[37320] = {"Jewelcrafting Special Order: A Fine Choker", 115992,170709,1,{[109118]=60}},
	[37324] = {"Out of Stock: Blackrock Ore", nil, nil, nil,{[109118]=20}},
	[37325] = {"Out of Stock: True Iron", nil, nil, nil,{[109119]=20}},
};
GH.jewelcraftingIDs = {
	["buildingID"] = 132,
}

function GH:isJewelcraftingEnabled()
	if type(GH.db.profile.jewelcraftingsupport)==nil then GH.db.profile.jewelcraftingsupport=true end
	return GH.db.profile.jewelcraftingsupport
end

function GH:isJewelcraftingQuest(qID)
	return GH.jewelcraftingquests[qID] and GH:isJewelcraftingEnabled() and GH:isSelectedJewelcraftingDaily(qID) 
end

function events.QUEST_DETAIL()
	if GH:isJewelcraftingQuest(GetQuestID()) then
		GH:RememberJewelcraftingDaily()
		if GH:QuestCanBeCompleted(GetQuestID()) then 
			AcceptQuest();
		end
	end
end

function GH:RememberJewelcraftingDaily()
	local realm = GetRealmName()
	GH.db.profile.jewelcraftingquest = GH.db.profile.jewelcraftingquest or {}
	GH.db.profile.jewelcraftingquest[realm] = GH.db.profile.jewelcraftingquest[realm] or {}
	GH.db.profile.jewelcraftingquest[realm].questID = GetQuestID()
	GH.db.profile.jewelcraftingquest[realm].validuntil = time()+GetQuestResetTime()
end

function GH:GetCurrentJewelcraftingDaily()
	local res = nil
	if GH.db.profile.jewelcraftingquest then
		local realm = GetRealmName()
		if GH.db.profile.jewelcraftingquest[realm] and GH.db.profile.jewelcraftingquest[realm].questID and GH.db.profile.jewelcraftingquest[realm].validuntil and GH.db.profile.jewelcraftingquest[realm].validuntil>time() then
			--print(GH.db.profile.jewelcraftingquest[realm].validuntil .. " > " .. time())
			res = GH.db.profile.jewelcraftingquest[realm].questID
		end
	end
	return res
end


function events.QUEST_ACCEPTED(qIndex, qID)
	if GH:isJewelcraftingQuest(GetQuestID()) then
		GH_craft()
	end
end

function events.QUEST_WATCH_UPDATE(qIndex)
	-- "At this point the game client's quest data is not yet updated, but will be after a subsequent QUEST_LOG_UPDATE event."
	GH.questwatchupdateindex = qIndex
end
function events.QUEST_LOG_UPDATE()
	if GH.questwatchupdateindex then
		local _, _, _, _, _, _, _, questID = GetQuestLogTitle(GH.questwatchupdateindex);
		GH.questwatchupdateindex = nil
		if GH:isJewelcraftingQuest(questID) then
			GH_craft()
		end
	end
end

function events.QUEST_COMPLETE()
	if GH:isJewelcraftingQuest(GetQuestID()) then
		GetQuestReward();
	end
end

function events.QUEST_PROGRESS()
	if GH:isJewelcraftingQuest(GetQuestID()) then
		if IsQuestCompletable() then
			CompleteQuest()
		else
			if GH:QuestCanBeCompleted(GetQuestID()) then 
				GH_craft()
			end
		end
	end
end
--fallback for not knowing jewelcrafting and talking to an NPC
function events.TRADE_SKILL_SHOW()
	if not GH:hasProfessionJewelcrafting() then
		GH_craft()
	end
end

-- register event listener
GarrisonHelper_Events:SetScript("OnEvent", function(self, event,...) events[event](...); end);
for event, func in pairs(events) do GarrisonHelper_Events:RegisterEvent(event); end

function GH:getProfessionOfTradeskillWindow()
	local link = GetTradeSkillListLink()
	if link then
		local _,_,professionSpellID,professionID,professionName =  string.find(link, "(%d+):(%d+)|?h?%[(.+)%]")
		return professionName,tonumber(professionID),tonumber(professionSpellID)
	end
end

function GH_craft()
	local questIndex, is, should, obj, sLink, questID
	--search active jc quest
	for questLogID=1,GetNumQuestLogEntries() do
		_, _, _, _, _, _, _, questID = GetQuestLogTitle(questLogID);
		if GH.jewelcraftingquests[questID] then questIndex=questLogID break end
	end
	if questIndex and GH.jewelcraftingquests[questID][2] then -- if quest was accepted and has something to craft
		
		local isComplete = true
		for i=1,GetNumQuestLeaderBoards(questIndex) do
			local _, _, finished = GetQuestObjectiveInfo(questID, i, false)--GetQuestObjectiveInfo(questID, index, displayComplete)
			isComplete = isComplete and finished
		end
		if isComplete then
			--quest objective reached, remove action button
			GH:destroy("jc")
			CloseQuest()
		else
			local sID = GH.jewelcraftingquests[questID][2]
			local spellID = GH.jewelcraftingquests[questID][3]
			local objective, sLink = GetItemInfo(sID)
			local num = GH.jewelcraftingquests[questID][4]
			
			local tooltiptext = ""
			if sLink then
				tooltiptext = tooltiptext .. L["to create"].. " " .. num .. " x " .. sLink .. "\n"..L["consumes"]..": "
			end
			tooltiptext = tooltiptext .. GH:JewelcraftingQuestResources(questID)
			if objective then
				--[[
					Workaround: This crafting needs no open tradeskill window. It picks up a crafting spell,
					places the spell somewhere on an empty action bar slot, uses the action and then clears
					the action bar slot
				]]
				--searches empty action bar slot
				--/run local emptySlot; for l=1, 120 do if not GetActionTexture(l) then emptySlot=l end end ClearCursor() PickupSpell(26925) PlaceAction(emptySlot) UseAction(emptySlot) PickupAction(emptySlot) ClearCursor()
				local emptySlot;
				for l=1, 120 do
					if not GetActionTexture(l) then emptySlot=l end -- get the last empty slot
				end
				local m
				if emptySlot then
					m = "/run ClearCursor() PickupSpell(" .. spellID .. ") PlaceAction(" .. emptySlot .. ") UseAction(" .. emptySlot .. ") PickupAction(" .. emptySlot .. ") ClearCursor() GH_destroy('jc')"
				else
					m = "/run CloseTradeSkill()\n/cast Jewelcrafting\n/run for i=1,GetNumTradeSkills() do if (GetTradeSkillInfo(i)=='" .. objective .. "') then DoTradeSkill(i) GH_destroy('jc') break end end CloseTradeSkill()"
				end
				if C_TradeSkillUI.IsNPCCrafting() then
					num = 1 -- I think it is risky to craft two items at once. what if you already crafted 1/2
					GH_CraftingNPCCanDoTradeSkill(objective,num,sID,tooltiptext)
				elseif GH:hasProfessionJewelcrafting() then
					GH:setActionButton(sID, m, tooltiptext,nil,nil,nil,"jc")
				else
					GH:print(L["Click the NPC to craft the quest objective!"])
				end
			end
		end
	end
end

--[[ 
	Called until the tradeskills of NPC are loaded.
	Then, it is looked, if he can help with creating the quest item.
]]
function GH_CraftingNPCCanDoTradeSkill(item,num,sID,tooltiptext,tryagain)
	tryagain = tryagain or true
	-- count errors
	GH.nojctradeskillcount = GH.nojctradeskillcount or 0
	if GetNumTradeSkills()==0 then
		--try again
		GH.nojctradeskillcount = GH.nojctradeskillcount+1
		if GH.nojctradeskillcount <= 25 then
			C_Timer.After(0.3, function() GH_CraftingNPCCanDoTradeSkill(item,num,sID,tooltiptext) end)
		else
			GH.nojctradeskillcount = 0
		end
	else
		local found = false
		GH.nojctradeskillcount = 0
		for i=1,GetNumTradeSkills() do
			if GetTradeSkillInfo(i)==item then
				--local m = "/run DoTradeSkill(" .. i .. "," .. num .. ")"
				-- we search again, because sometimes the wrong item was craftet
				local m = "/run for i=1,GetNumTradeSkills() do if GetTradeSkillInfo(i)== '" .. item .. "' then DoTradeSkill(i, "..num..") break end end"
				GH:setActionButton(sID, m, tooltiptext,nil,nil,nil,"jc")
				found = true
				break
			end
		end
		if tryagain and not found then
			C_Timer.After(0.5, function() GH_CraftingNPCCanDoTradeSkill(item,num,sID,tooltiptext,false) end)
		end
	end
end

function GH:JewelcraftingQuestResources(questID)
	local res = ""
	for item, numItems in pairs(GH.jewelcraftingquests[questID][5]) do
		if #res>0 then res = res .. ", " end
		local v = select(2,GetItemInfo(item))
		res = res .. numItems .. " x " .. tostring(v)
	end
	return res
end

function GH:JewelcraftingQuestTooltip(questID,highlight)
	local res = ""	
	for item, numItems in pairs(GH.jewelcraftingquests[questID][5]) do
		if #res>0 then res = res .. ", " end
		local itemTexture = select(10,GetItemInfo(item))
		if highlight then
			res = res .. (numItems <= GetItemCount(item, true) and ("|cff00ff00"..numItems.."|r") or  ("|cffff0000"..numItems.."|r") ) .. " x " .. "\124T" .. itemTexture  ..":12\124t"
		else
			res = res .. numItems .. " x " .. "\124T" .. itemTexture  ..":36\124t"
		end
	end
	return res
end

--[[
	check if required materials for completing or crafting is available
]]
function GH:QuestCanBeCompleted(questID)
	local res = true
	if GH:isJewelcraftingQuest(questID) then
		for item, numItems in pairs(GH.jewelcraftingquests[questID][5]) do
			-- check if required items are in inventory (and bank)
			if numItems>GetItemCount(item, true) then
				GH:print(L["Not enough reagents"].." ("..GetItemInfo(item).." "..GetItemCount(item, true).."/"..numItems..")");
				if GH.db.profile.sound then
					PlaySound("igQuestFailed")
				end
				res = false
			end
		end
	end
	return res
end	

function GH:isSelectedJewelcraftingDaily(qID)
	local res = true
	for i=1,10 do
		if GH.db.profile["jewelcraftingquest" .. i .. "id"]==qID then
			-- if nil or true -> yes: no otherwise
			if type(GH.db.profile["jewelcraftingquest" .. i])=="boolean" and not GH.db.profile["jewelcraftingquest" .. i] then
				res = false
				break
			end
		end
	end
	return res
end

function GH:hasProfessionJewelcrafting()
	local res = false
	local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions("player");
	local skillLevel
	if prof1 then
		skillLine = select(7,GetProfessionInfo(prof1))
	end
	if skillLine == 755 then
		res = true
	end
	if not res and prof2 then
		skillLine = select(7,GetProfessionInfo(prof2))
		if skillLine == 755 then
			res = true
		end
	end
	return res
end

function GH:GetJewelcraftingTooltip()
	if GH:isJewelcraftingEnabled() and GH:hasBuilding(GH.jewelcraftingIDs["buildingID"],2) then
		local buildingName = select(2,C_Garrison.GetBuildingInfo(GH.jewelcraftingIDs["buildingID"]))
		local tip = GH.tip
		tip:AddLine(" ")
		tip:AddHeader("|cffF5DA81"..tostring(buildingName))
		local done = false
		for questID, _ in pairs(GH.jewelcraftingquests) do
			done = done or IsQuestFlaggedCompleted(questID)
		end
		
		tip:AddLine("   "..tostring(buildingName).." "..L["daily quest"], (done and "|cff00ff00"..L["completed"].."|r" or "|cffff0000"..L["not completed"].."|r") );
		if GH.db.profile.showdailyhelp then
			tip:AddLine(" ", (GH:hasBuilding(GH.jewelcraftingIDs["buildingID"],2,true) and "|cff00ff00"..L["follower assigned"].."|r" or "|cffff0000"..L["no follower assigned"].."|r\n"));
			
			if not done then
				
				local currDaily = GH:GetCurrentJewelcraftingDaily() -- if toon or another toon on the same server has seen the daily
				if currDaily then
					if GH:isSelectedJewelcraftingDaily(currDaily) then
						tip:AddLine("   "..L["Can the quest be completed?"], GH:JewelcraftingQuestTooltip(currDaily,true));
					end
				else
					local q = "   "..L["Can each quest be completed?"]
					for questID, _ in pairs(GH.jewelcraftingquests) do
						if GH:isSelectedJewelcraftingDaily(questID) then
							tip:AddLine(q, GH:JewelcraftingQuestTooltip(questID,true));
							q = " "
						end
					end
				end
			end
		end
	end
end

function GH:GetJewelcraftingOpenTaskCount()
	local openTask = 0
	local tasks = 0
	local neededmats
	if GH:hasBuilding(GH.jewelcraftingIDs["buildingID"],2,true) then
		if GH:isJewelcraftingEnabled() then
			local done = false
			for questID, _ in pairs(GH.jewelcraftingquests) do
				done = done or IsQuestFlaggedCompleted(questID)
			end
			if not done then
				openTask = 1
			end
			
			local currDaily = GH:GetCurrentJewelcraftingDaily() -- if toon or another toon on the same server has seen the daily
			if currDaily and GH:isSelectedJewelcraftingDaily(currDaily) and not IsQuestFlaggedCompleted(currDaily) then
				neededmats = {}
				for item, numItems in pairs(GH.jewelcraftingquests[currDaily][5]) do
					if numItems > GetItemCount(item, true) then
						neededmats[item] = numItems - GetItemCount(item, true)
					end
				end
			end
		end
		tasks = 1
	end
	return openTask, tasks, neededmats
end

function GH:JewelcraftingWelcome()
	if GH:isJewelcraftingEnabled() and GH:hasBuilding(GH.jewelcraftingIDs["buildingID"],2,true) then
		local done = false
		for questID, _ in pairs(GH.jewelcraftingquests) do
			done = done or IsQuestFlaggedCompleted(questID)
		end
		
		if not done then
			local buildingName = select(2,C_Garrison.GetBuildingInfo(GH.jewelcraftingIDs["buildingID"]))
			GH:print(tostring(buildingName) .. " " .. L["daily quest"] .." " .. L["not completed today."]);
		end
	end
end

--[[
	Needed, because if cache is empty, this information has to be retrieved from server.
]]
function GH:PreLoadJewelcraftingItemInfos()
	for questID, info in pairs(GH.jewelcraftingquests) do
		for item, _ in pairs(GH.jewelcraftingquests[questID][5]) do
			GetItemInfo(item)
		end
		if GH.jewelcraftingquests[questID][2] then
			GetItemInfo(GH.jewelcraftingquests[questID][2])
		end
		if GH.jewelcraftingquests[questID][3] then
			GetItemInfo(GH.jewelcraftingquests[questID][3])
		end
	end
	C_Garrison.GetBuildingInfo(GH.jewelcraftingIDs["buildingID"])
end
GH:PreLoadJewelcraftingItemInfos() -- TODO move this to an event like addon loaded