local GH = GarrisonHelper
local L = GH.L
-- Frame and Events
local GarrisonHelper_Events = CreateFrame("Frame");
local events = {};
GH.scribeIDs = {
	["Partial Receipt"] = 119126,
	["Milling"] = 51005,
	["Milling Icon"] = 114942,
	["Cerulean Pigment"] = 114931,
	["Light Parchment"] = 39354,
	["buildingID"] = 130,
}

local millingPriority = {109125,109124,109128,109127,109129,109126}

function events.TRADE_SKILL_SHOW()
	--print("TRADE_SKILL_SHOW " .. tostring(GH.inprogress))
	GH:doMillingDaily()
end
--[[
function events.UPDATE_TRADESKILL_RECAST()
	print("UPDATE_TRADESKILL_RECAST " .. tostring(GH.inprogress))
	-- may happen when trade skill was not loaded when showing
	GH:doMillingDaily()
end
]]
--[[
function events.UNIT_SPELLCAST_SUCCEEDED(self, event, target, spell)
	print("UNIT_SPELLCAST_SUCCEEDED " .. spell)
	-- if milling was performed
	if spell and spell == "Milling" then
		waitFor = true
	end
end
]]
function events.BAG_UPDATE()
	--print("BAG_UPDATE " .. tostring(GH.inprogress))
	-- if milling was performed, then pigments have been looted
	GH:doMillingDaily()
end

function events.BAG_UPDATE_DALAYED()
	--print("BAG_UPDATE_DALAYED " .. tostring(GH.inprogress))
	-- if milling was performed, then pigments have been looted
	GH:doMillingDaily()
end

function GH:doMillingDaily()
	--[[Trade skill windows was opened. Check if own or NPC]]--
	-- count errors
	GH.notradeskillcount = GH.notradeskillcount or 0
	--avoid several calls (from several events) at once
	if type(GH.inprogress)==nil then
		GH.inprogress = time()
	elseif GH.inprogress == time() then
		return
	end
	--local name = GetTradeSkillLine(); --XXX language dependent, where is the id
	--GH:print("name "..tostring(name))
	if GH:isInscriptionEnabled() and C_TradeSkillUI.IsNPCCrafting() then
		--print("GetNumTradeSkills() " .. GetNumTradeSkills())
		if GetNumTradeSkills() == 0 then
			--try again
			GH.notradeskillcount = GH.notradeskillcount+1
			GH.inprogress =0 -- reset, that the next call will not be blocked
			if GH.notradeskillcount <= 10 then
				C_Timer.After(0.5, events.TRADE_SKILL_SHOW)
			else
				GH.notradeskillcount = 0
			end
		else
			GH.notradeskillcount = 0
			GH:destroy("scribe")
			for i=1,GetNumTradeSkills() do
				local sLink = GetTradeSkillItemLink(i)
				if sLink then
					local sID  = GH:getIDFromLink(sLink)
					if sID and sID == GH.scribeIDs["Partial Receipt"] and not GetTradeSkillCooldown(i) then
						local skillName, skillType, numAvailable = GetTradeSkillInfo(i)
						-- print(skillName .. "  "  ..  skillType .. "  "  ..  numAvailable)
						--print(GH:getIDFromLink(sLink))
						--if numAvailable < 1 then
						-- unfortunately we cannot take numAvailable. this value will not be updated
						if GetItemCount(GH.scribeIDs["Cerulean Pigment"], true)<4 then
							if GH:hasProfessionInscription() then
								MillingCommand()
							else
								GH:print(L["Ask somebody to mill herbs for you."])
							end
						elseif GetItemCount(GH.scribeIDs["Light Parchment"], true)<2 then
							GH:print(L["Get at least 2 x"] .. " " .. select(2,GetItemInfo(GH.scribeIDs["Light Parchment"])))
						else
							--DoTradeSkill(i)
							GH:setActionButton(sID,"/run DoTradeSkill(" .. i .. ")", L["to create"] .. " " .. skillName,nil,nil,nil,"scribe")
						end
					end
				end
			end
		end
	end
end

function GH:hasProfessionInscription()
local res = false
	local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions("player");
	local skillLevel
	if prof1 then
		skillLine = select(7,GetProfessionInfo(prof1))
	end
	if skillLine == 773 then
		res = true
	end
	if not res and prof2 then
		skillLine = select(7,GetProfessionInfo(prof2))
		if skillLine == 773 then
			res = true
		end
	end
	return res
end


--[[
used for milling macro:

#showtooltip Milling
/run MillingMacro()
/click GH_Button
]]
function MillingMacro()
	MillingCommand(true)
end
--[[
#showtooltip Milling
/mill
/click GH_Button
]]
--SLASH_MILLING1 = "/mill" SlashCmdList["MILLING"]=function(m)MillingMacro()end


function MillingCommand(hideButton)
	local herb = nil;
	local mp = GH:MillingPriority()
	if #mp>0 then millingPriority=mp end
	--find herb in bag
	for j=1,#millingPriority do
		if GetItemCount(millingPriority[j], false)>=5 then
			herb = millingPriority[j]
			-- we have found at least 5 of one herb, but we have to make sure it's a stack of 5	
			
			local b = false
			for bag=0,4 do
				for slot=1,GetContainerNumSlots(bag) do
					local itemID = GetContainerItemID(bag, slot)
					if itemID and itemID == herb then
						local _, count = GetContainerItemInfo(bag, slot)
						-- we have found the first stack of this herb, the macro will always mill the first stack
						if count >= 5 then
							-- everything is fine
						else
							GH:print(L["There are"].." "..GetItemCount(id, false).." x "..herb..L[", but the first stack is too small to mill!"]);
							herb = nil
						end
						b =true
					end
					if b then break end
				end
				if b then break end
			end
			if herb then break end
		end
	end
	if herb then
		if hideButton and ((not GH.millingPrinted or GH.millingPrinted + 60 < time()) or (GH.millingHerb and GH.millingHerb ~= herb)) then 
			GH.millingPrinted = time()
			GH.millingHerb = herb
			GH:print(L["Milling"] .. " "..GetItemInfo(herb))
		end
		--print("|r|cFFFF7700<GarrisonHelper>|r|cFFFFFFFF Press button to mill " .. herb);
		GH:setActionButton(GH.scribeIDs["Milling"],"/cast "..GetSpellInfo(GH.scribeIDs["Milling"]).."\n/use item:" .. herb,L["to mill"].." " .. GetItemInfo(herb), true, GH.scribeIDs["Milling Icon"],hideButton,"scribe")
	else
		GH:print(L["Not enough reagents"].." "..L["and no herbs to mill found."]);
		if GH.db.profile.sound then
			PlaySound("igQuestFailed")
		end
	end
end

function GH:MillingPriority()
	--load saved herb list
	local res = {}
	for i=1,10 do
		if GH.db.profile["herb" .. i] then
			res[#res+1]=tonumber(GH.db.profile["herb" .. i])
		end
	end
	return res
end

function GH:GetScribeTooltip()
	if GH:isInscriptionEnabled() and GH:hasBuilding(GH.scribeIDs["buildingID"],2) then
		local buildingName = select(2,C_Garrison.GetBuildingInfo(GH.scribeIDs["buildingID"]))
		local tip = GH.tip
		tip:AddLine(" ")
		tip:AddHeader("|cffF5DA81"..tostring(buildingName))
		local _,duration = GetSpellCooldown(176513)
		tip:AddLine("   "..tostring(buildingName).." "..L["daily quest"], (duration~=0 and "|cff00ff00"..L["completed"].."|r" or "|cffff0000"..L["not completed"].."|r") );
		if GH.db.profile.showdailyhelp then
			tip:AddLine(" ", (GH:hasBuilding(GH.scribeIDs["buildingID"],2,true) and "|cff00ff00"..L["follower assigned"].."|r" or "|cffff0000"..L["no follower assigned"].."|r\n"));
		end
	end
end

function GH:GetScribeOpenTaskCount()
	local openTask = 0
	local tasks = 0
	if GH:hasBuilding(GH.scribeIDs["buildingID"],2,true) then
		if GH:isInscriptionEnabled() and GetSpellCooldown(176513)==0 then
			openTask = 1
		end
		tasks = 1
	end
	return openTask, tasks
end

function GH:ScribeWelcome()
	if GH:isInscriptionEnabled() and GH:hasBuilding(GH.scribeIDs["buildingID"],2,true) then
		local _,duration = GetSpellCooldown(176513)
		if duration==0 then
			local buildingName = select(2,C_Garrison.GetBuildingInfo(GH.scribeIDs["buildingID"]))
			GH:print(tostring(buildingName) .. " " .. L["daily quest"] .." " .. L["not completed today."]);
		end
	end
end

function GH:isInscriptionEnabled()
	if type(GH.db.profile.scribesupport)==nil then GH.db.profile.scribesupport=true end
	return GH.db.profile.scribesupport
end

-- register event listener
GarrisonHelper_Events:SetScript("OnEvent", function(self, event, ...) events[event](self, event, ...); end);
for event, func in pairs(events) do GarrisonHelper_Events:RegisterEvent(event); end


--[[
	Needed, because if cache is empty, this information have to be retrieved from server.
]]
function GH:PreLoadInscriptionItemInfos()
	for herb, id in pairs(millingPriority) do
		GetItemInfo(id)
	end
	C_Garrison.GetBuildingInfo(GH.scribeIDs["buildingID"])
	GetSpellInfo(GH.scribeIDs["Milling"])
	GetItemInfo(GH.scribeIDs["Light Parchment"])
end
GH:PreLoadInscriptionItemInfos() -- TODO move this to an event like addon loaded