local GH = GarrisonHelper
-- Frame and Events
local GarrisonHelper_Events = CreateFrame("Frame");
local events = {};
local L = GH.L

function GH:isWorkOrdersEnabled()
	if type(GH.db.profile.workorderssupport)==nil then GH.db.profile.workorderssupport=false end -- default is false!
	return GH.db.profile.workorderssupport
end

function events.SHIPMENT_CRAFTER_INFO(uiTab, currWorkOrders, maxWorkOrders, _, plotID)
	local buildingID  = C_Garrison.GetOwnedBuildingInfo(plotID)
	if buildingID then
		if GH.db.profile.workordersbuildingenabled and GH.db.profile.workordersbuildingenabled[GH:buildingHash(buildingID)] and GarrisonCapacitiveDisplayFrame then
			if GH:buildingHash(buildingID)==111 then --trading post
				for i = 1, C_Garrison.GetNumShipmentReagents() do
					local name, texture, quality, needed, quantity, itemID = C_Garrison.GetShipmentReagentInfo(i);
					GH.db.profile.tradingpostworkorder = {}
					GH.db.profile.tradingpostworkorder.itemID = itemID
					GH.db.profile.tradingpostworkorder.itemName  = name
					GH.db.profile.tradingpostworkorder.validuntil = time()+GetQuestResetTime()
					if not GH.db.profile.workorderstradegood[itemID] then
						-- trade good not selected in /gh
						return
					end
				end
			end
			local b = GarrisonCapacitiveDisplayFrame.CreateAllWorkOrdersButton
			if b and b:IsVisible() and b:IsEnabled() and currWorkOrders < maxWorkOrders and (not GH.CreateAllWorkOrdersButtonPressedTime or GH.CreateAllWorkOrdersButtonPressedTime+3<GetTime()) then 
				b:Click()
				GH.CreateAllWorkOrdersButtonPressedTime = GetTime() -- do not press the same button twice (a bug may result in loosing trade goods)
			end
		end
	end
end

GameTooltip:HookScript("OnShow", function(self) GH:WorkOrderExtendedTooltip(self) end)
--GameTooltip:HookScript("OnTooltipCleared", function(self) GH:WorkOrderExtendedTooltip(self) end)
GameTooltip:HookScript("OnUpdate", function(self) GH:WorkOrderExtendedTooltip(self) end)
function GH:WorkOrderExtendedTooltip(self)
	if  GH.db.profile.workordertooltip and
		not UnitExists("mouseover") and
		GH and
		(GH:InsideGarrison() or	GH:InsideShipyard()) and
		GetMouseFocus() and
		GetMouseFocus():GetName() and
		GetMouseFocus():GetName()=="WorldFrame" and
		GameTooltip and
		GameTooltip:IsShown() and
		GameTooltipTextLeft1 and
		GameTooltipTextLeft1:GetText() then
		
		for i=1,GameTooltip:NumLines() do
			if _G["GameTooltipTextLeft"..i] and _G["GameTooltipTextLeft"..i]:GetText() and string.find(_G["GameTooltipTextLeft"..i]:GetText(),"GarrisonHelper") then
				-- tooltip was handled already
				return
			end
		end
		
		local name, texture, shipmentCapacity, shipmentsReady, shipmentsTotal, creationTime, duration, timeleftString, itemName, itemIcon, itemQuality, itemID
		
		
		if string.find(GameTooltipTextLeft1:GetText(), GARRISON_CACHE) then
			--[[
				We found a tooltip of an object in the world frame where the title includes the term GARRISON_CACHE in your language.
			]]
			local gr =  GH:getGarrisonCache()
			if gr then
				local cachesize = GH:GetCacheSize()
				local grCurrency = select(2,GetCurrencyInfo("currency:824"))
				GameTooltip:AddLine("\n|r|cFFFF7700<GarrisonHelper>|r:")
				if (grCurrency+gr) > 10000 then
					GameTooltip:AddLine("|r|cFFFF0000"..L["Don't take the cache. Spend your garrison resources first!"].."|r")
					GameTooltip:AddLine("|r|cFFFF0000".."("..grCurrency.." + "..gr.." > 10000)|r")
				end
				
				GameTooltip:AddLine(GH:GetWorkOrderString(gr,gr,cachesize, 250))
				GameTooltip:AddDoubleLine(select(1,GetCurrencyInfo(GH.tradingpostIDs["garrisonResourcesId"])), gr.."/"..cachesize)
				local t = (cachesize-gr) * 600
				if t > 0 then
					local minutes = floor(t/60)
					local hours = nil
					if minutes > 120 then
						hours = floor(minutes/60)
					end
					GameTooltip:AddDoubleLine(" ", "(".. (hours and ("|cffF5DA81" .. hours .."|r h" ) or ("|cffF5DA81" .. minutes .."|r min")) .. " "..L["left"]..")")
				end
			end
			GameTooltip:Show();
		else
			local buildings = C_Garrison.GetBuildings(LE_GARRISON_TYPE_6_0)
			
			for i = 1, #buildings do
				name, texture, shipmentCapacity, shipmentsReady, shipmentsTotal, creationTime, duration, timeleftString, itemName, itemIcon, itemQuality, itemID = C_Garrison.GetLandingPageShipmentInfo(buildings[i].buildingID);
				
				-- work orders for profession buildigs do not include the building name but the profession name
				if GH.buildings2skill[GH:buildingHash(buildings[i].buildingID)] then
					name = select(1,GetSpellInfo(GH.buildings2skill[GH:buildingHash(buildings[i].buildingID)][2]))
				elseif GH.buildings2localisedWorkOrder[GH:buildingHash(buildings[i].buildingID)] and GH.buildings2localisedWorkOrder[GH:buildingHash(buildings[i].buildingID)][GetLocale()] then
					-- unfortunaltely for some strings there are no localised constants
					if type(GH.buildings2localisedWorkOrder[GH:buildingHash(buildings[i].buildingID)][GetLocale()])=="string" then
						name = GH.buildings2localisedWorkOrder[GH:buildingHash(buildings[i].buildingID)][GetLocale()]
					elseif GH.buildings2localisedWorkOrder[GH:buildingHash(buildings[i].buildingID)][GetLocale()][UnitFactionGroup("player")] then
						name = GH.buildings2localisedWorkOrder[GH:buildingHash(buildings[i].buildingID)][GetLocale()][UnitFactionGroup("player")]
					end
				elseif GH:buildingHash(buildings[i].buildingID) == 205 then --shipyard
					--print("#1#")
					-- work order tooltip contains ship class
					for ship, sample in pairs(GH.ships) do
						--print("#2#"..C_Garrison.GetFollowerInfo(sample).className)
						if C_Garrison.GetFollowerInfo(sample) and C_Garrison.GetFollowerInfo(sample).className and string.find(GameTooltipTextLeft1:GetText(), C_Garrison.GetFollowerInfo(sample).className) then
							name = C_Garrison.GetFollowerInfo(sample).className
							--print("#3#"..name)
							break
						end
					end
				end
				
				if name and string.find(GameTooltipTextLeft1:GetText(), name) then
					--[[
						We found a tooltip of an object in the world frame where the title belongs to a building.
						I assume, that's the work order crate. Hopefully that's true for all languages.
					]]
					shipmentsReady = shipmentsReady or 0
					shipmentsTotal = shipmentsTotal or 0
					creationTime = creationTime or 0
					duration = duration or 0
					if shipmentCapacity then
						GameTooltip:AddLine("\n|r|cFFFF7700<GarrisonHelper>|r:")
						if shipmentsTotal > 1 then
							GameTooltip:AddLine(GH:GetWorkOrderString(shipmentsReady,shipmentsTotal,shipmentCapacity, 250))
						end
						GameTooltip:AddDoubleLine(L["ready"] .. ":", shipmentsReady .. " / " .. shipmentsTotal)
						
						-- building specific additions
						if GH:buildingHash(buildings[i].buildingID) == 111 and shipmentsReady > 0 then
							-- trading post
							GameTooltip:AddDoubleLine(select(1,GetCurrencyInfo(GH.tradingpostIDs["garrisonResourcesId"])), shipmentsReady*30)
						elseif GH:buildingHash(buildings[i].buildingID) == 8 then
							-- armory
							local info = GH:GetArmoryTooltipExtension()
							for i=1,#info do
								GameTooltip:AddDoubleLine(info[i].left,  info[i].right);
							end
						end
						
						local t = creationTime-time()+((shipmentsTotal-shipmentsReady)*duration)
												
						if t > 0 then
							local minutes = floor(t/60)
							local hours = nil
							if minutes > 120 then
								hours = floor(minutes/60)
							end
							GameTooltip:AddDoubleLine(" ", "(".. (hours and ("|cffF5DA81" .. hours .."|r h" ) or ("|cffF5DA81" .. minutes .."|r min")) .. " "..L["left"]..")")
						end
						if shipmentCapacity > shipmentsTotal then
							GameTooltip:AddDoubleLine(L["unused work orders"] .. ":", shipmentCapacity-shipmentsTotal)
						end
						GameTooltip:Show();
					end
					break
				end
			end
		end
	end
end

function GH:GetWorkOrderString(ready,total,capacity, lengthInPixel)
	lengthInPixel = lengthInPixel or 200
	local res = ""
	--print(((100*ready)/capacity))
	--print(((100*(total-ready))/capacity))
	--print(((100*(capacity-total))/capacity))
	if ready > 0 then
		res = res .. "\124T".. "Interface\\AddOns\\GarrisonHelper\\img\\green" ..":6:" ..((lengthInPixel*ready)/capacity).."\124t"
	end
	if (total-ready) > 0 then
		res = res .. "\124T".. "Interface\\AddOns\\GarrisonHelper\\img\\yellow" ..":6:" ..(lengthInPixel*((total-ready)/capacity)).."\124t"
	end
	if capacity~=total then
		res = res .. "\124T".. "Interface\\AddOns\\GarrisonHelper\\img\\grey" ..":6:" ..(lengthInPixel*((capacity-total))/capacity).."\124t"
	end
	return res
end

function events.SHOW_LOOT_TOAST(typeIdentifier, itemLink, quantity, specID, sex, isPersonal, lootSource)
	--[[
		print("typeIdentifier" .. " " .. tostring(typeIdentifier))
		print("itemLink" .. " " .. tostring(itemLink))
		print("quantity" .. " " .. tostring(quantity))
		print("specID" .. " " .. tostring(specID))
		print("sex" .. " " .. tostring(sex))
		print("isPersonal" .. " " .. tostring(isPersonal))
		print("lootSource" .. " " .. tostring(lootSource))
	]]
	if typeIdentifier == "currency" and isPersonal and lootSource == 10 and itemLink:match("currency:824") then
		GH.db.profile.garrisoncache = GH.db.profile.garrisoncache or {}
		GH.db.profile.workordersbuildingenabled[GH:Player()] = time()
	end
end


-- register event listener
GarrisonHelper_Events:SetScript("OnEvent", function(self, event, ...) events[event](...); end);
for event, func in pairs(events) do GarrisonHelper_Events:RegisterEvent(event); end


function GH:getGarrisonCache()
	GH.db.profile.garrisoncache = GH.db.profile.garrisoncache or {}
	local timestamp = GH.db.profile.workordersbuildingenabled[GH:Player()]
	local cachesize = GH:GetCacheSize()
	
	if timestamp then 
		return math.min(cachesize, math.floor(((time() - timestamp) / 60) / 10))
	else 
		return nil
	end
end


function GH:GetCacheSize()
	local res = 500
	if IsQuestFlaggedCompleted(37485) then
		res = 1000 
--	elseif IsQuestFlaggedCompleted(37935) or IsQuestFlaggedCompleted(38445) then
--		res = 750
	end
	return res
end

function GH:WorkOrdersBuilding(buildingID)
	local res = ""
	local buildingName = select(2,C_Garrison.GetBuildingInfo(buildingID))
	res = buildingName;
	return res
end

function GH:WorkOrdersBuildingTooltip(buildingID)
	local res = ""
	local description = select(5,C_Garrison.GetBuildingInfo(buildingID))
	res = description;
	return res
end

function GH:WorkOrdersTradegood(itemID)
	local res = ""
	local itemName = GetItemInfo(itemID)
	res = itemName;
	return res
end

function GH:WorkOrdersTradegoodTooltip(itemID)
	local res = ""
	local itemTexture = select(10,GetItemInfo(itemID))
	res = "\124T" .. itemTexture  ..":24\124t";
	return res
end


--[[
	Needed, because if cache is empty, this information have to be retrieved from server.
]]
function GH:PreLoadWorkorderItemInfos()
	--for i=1,#millingPriority do
	--	GetItemInfo(millingPriority[i])
	--end
end
GH:PreLoadWorkorderItemInfos() -- TODO move this to an event like addon loaded